var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Color3, MeshBuilder } from '@babylonjs/core';
import * as Inputs from '../inputs/inputs';
import { GridMaterial } from '@babylonjs/materials';
export class Draw {
    constructor(
    /**
     * @ignore true
     */
    point, 
    /**
     * @ignore true
     */
    line, 
    /**
     * @ignore true
     */
    polyline, 
    /**
     * @ignore true
     */
    node, 
    /**
     * @ignore true
     */
    verbCurve, 
    /**
     * @ignore true
     */
    verbSurface, 
    /**
     * @ignore true
     */
    jscad, 
    /**
     * @ignore true
     */
    occt, 
    /**
     * @ignore true
     */
    tag, 
    /**
     * @ignore true
     */
    context) {
        this.point = point;
        this.line = line;
        this.polyline = polyline;
        this.node = node;
        this.verbCurve = verbCurve;
        this.verbSurface = verbSurface;
        this.jscad = jscad;
        this.occt = occt;
        this.tag = tag;
        this.context = context;
        this.defaultBasicOptions = {
            colours: '#ff0000',
            updatable: false,
            hidden: false,
            opacity: 1,
            size: 6,
        };
        this.defaultNodeOptions = {
            colorX: '#ff0000',
            colorY: '#00ff00',
            colorZ: '#0000ff',
            size: 2,
        };
    }
    /**
     * Draws any kind of geometry after all input promises are resolved. Inputs can also be non-promise like.
     * @param inputs Contains options and entities to be drawn
     * @returns BabylonJS Mesh Promise
     * @group draw
     * @shortname draw anything
     * @disposableOutput true
     */
    drawAnyAsync(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            const entity = inputs.entity;
            // we start with async ones
            if (this.detectJscadMesh(entity)) {
                return this.handleJscadMesh(inputs);
            }
            if (this.detectOcctShape(entity)) {
                return this.handleOcctShape(inputs);
            }
            else if (this.detectJscadMeshes(entity)) {
                return this.handleJscadMeshes(inputs);
            }
            else {
                // here we have all sync drawer functions
                return Promise.resolve(this.drawAny(inputs));
            }
        });
    }
    updateAny(inputs) {
        let result;
        if (inputs.babylonMesh && inputs.babylonMesh.metadata) {
            const type = inputs.babylonMesh.metadata.type;
            switch (type) {
                case Inputs.Draw.drawingTypes.point:
                    result = this.handlePoint(inputs);
                    break;
                case Inputs.Draw.drawingTypes.points:
                    result = this.handlePoints(inputs);
                    break;
                case Inputs.Draw.drawingTypes.line:
                    result = this.handleLine(inputs);
                    break;
                case Inputs.Draw.drawingTypes.lines:
                    result = this.handleLines(inputs);
                    break;
                case Inputs.Draw.drawingTypes.polyline:
                    result = this.handlePolyline(inputs);
                    break;
                case Inputs.Draw.drawingTypes.polylines:
                    result = this.handlePolylines(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbCurve:
                    result = this.handleVerbCurve(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbCurves:
                    result = this.handleVerbCurves(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbSurface:
                    result = this.handleVerbSurface(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbSurfaces:
                    result = this.handleVerbSurfaces(inputs);
                    break;
                case Inputs.Draw.drawingTypes.tag:
                    result = this.handleTag(inputs);
                    break;
                case Inputs.Draw.drawingTypes.tags:
                    result = this.handleTags(inputs);
                    break;
                case Inputs.Draw.drawingTypes.node:
                    result = this.handleNode(inputs);
                    break;
                case Inputs.Draw.drawingTypes.nodes:
                    result = this.handleNodes(inputs);
                    break;
                default:
                    break;
            }
        }
        return result;
    }
    /**
     * Draws a grid mesh on the ground plane in 3D space. This helps to orient yourself in the world.
     * @param inputs Describes various parameters of the grid mesh like size, colour, etc.
     * @group draw
     * @shortname draw grid
     * @disposableOutput true
     */
    drawGridMesh(inputs) {
        try {
            const groundMaterial = new GridMaterial(`groundMaterial${Math.random()}`, this.context.scene);
            groundMaterial.majorUnitFrequency = inputs.majorUnitFrequency;
            groundMaterial.minorUnitVisibility = inputs.minorUnitVisibility;
            groundMaterial.gridRatio = inputs.gridRatio;
            groundMaterial.backFaceCulling = inputs.backFaceCulling;
            groundMaterial.mainColor = Color3.FromHexString(inputs.mainColor);
            groundMaterial.lineColor = Color3.FromHexString(inputs.secondaryColor);
            groundMaterial.opacity = inputs.opacity;
            const ground = MeshBuilder.CreateGround(`ground${Math.random()}`, {
                width: inputs.width,
                height: inputs.height,
                subdivisions: inputs.subdivisions,
                updatable: false,
            }, this.context.scene);
            ground.material = groundMaterial;
            return ground;
        }
        catch (e) {
            console.log('Error happened: ', e);
            return MeshBuilder.CreateBox('error-ground', { size: 0.00000001 }, this.context.scene);
        }
    }
    /**
     * Draws any kind of geometry. Inputs can not be promises.
     * @param inputs Contains options and entities to be drawn
     * @returns BabylonJS Mesh
     */
    drawAny(inputs) {
        let result;
        const entity = inputs.entity;
        if (!inputs.babylonMesh) {
            if (this.detectLine(entity)) {
                result = this.handleLine(inputs);
            }
            else if (this.detectPoint(entity)) {
                result = this.handlePoint(inputs);
            }
            else if (this.detectPolyline(entity)) {
                result = this.handlePolyline(inputs);
            }
            else if (this.detectNode(entity)) {
                result = this.handleNode(inputs);
            }
            else if (this.detectVerbCurve(entity)) {
                result = this.handleVerbCurve(inputs);
            }
            else if (this.detectVerbSurface(entity)) {
                result = this.handleVerbSurface(inputs);
            }
            else if (this.detectPolylines(entity)) {
                result = this.handlePolylines(inputs);
            }
            else if (this.detectLines(entity)) {
                result = this.handleLines(inputs);
            }
            else if (this.detectPoints(entity)) {
                result = this.handlePoints(inputs);
            }
            else if (this.detectNodes(entity)) {
                result = this.handleNodes(inputs);
            }
            else if (this.detectVerbCurves(entity)) {
                result = this.handleVerbCurves(inputs);
            }
            else if (this.detectVerbSurfaces(entity)) {
                result = this.handleVerbSurfaces(inputs);
            }
            else if (this.detectTag(entity)) {
                result = this.handleTag(inputs);
            }
            else if (this.detectTags(entity)) {
                result = this.handleTags(inputs);
            }
        }
        else {
            // here types are marked on mesh metadata
            result = this.updateAny(inputs);
        }
        return result;
    }
    /**
     * Creates draw options for basic geometry types like points, lines, polylines, surfaces and jscad meshes
     * @param inputs option definition
     * @returns options
     * @group options
     * @shortname simple
     */
    optionsSimple(inputs) {
        return inputs;
    }
    /**
     * Creates draw options for occt shape geometry like edges, wires, faces, shells, solids and compounds
     * @param inputs option definition
     * @returns options
     * @group options
     * @shortname occt shape
     */
    optionsOcctShape(inputs) {
        return inputs;
    }
    /**
     * Creates draw options for babylon js nodes
     * @param inputs option definition
     * @returns options
     * @group options
     * @shortname babylon node
     */
    optionsBabylonNode(inputs) {
        return inputs;
    }
    handleTags(inputs) {
        const options = inputs.options ? inputs.options : {
            updatable: false,
        };
        let result = this.tag.drawTags(Object.assign({ tagsVariable: inputs.babylonMesh, tags: inputs.entity }, options));
        result.metadata = { type: Inputs.Draw.drawingTypes.tags, options };
        return result;
    }
    handleTag(inputs) {
        let options = inputs.options ? inputs.options : {
            updatable: false,
        };
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.tag.drawTag(Object.assign({ tagVariable: inputs.babylonMesh, tag: inputs.entity }, options));
        result.metadata = { type: Inputs.Draw.drawingTypes.tag, options };
        return result;
    }
    handleVerbSurfaces(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.verbSurface.drawSurfacesMultiColour(Object.assign({ surfacesMesh: inputs.babylonMesh, surfaces: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.verbSurfaces, options, result);
        return result;
    }
    handleVerbCurves(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.verbCurve.drawCurves(Object.assign({ curvesMesh: inputs.babylonMesh, curves: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.verbCurves, options, result);
        return result;
    }
    handleNodes(inputs) {
        let options = inputs.options ? inputs.options : this.defaultNodeOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = inputs.entity;
        this.node.drawNodes(Object.assign({ nodes: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.nodes, options, result);
        return result;
    }
    handlePoints(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.point.drawPoints(Object.assign({ pointsMesh: inputs.babylonMesh, points: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.points, options, result);
        return result;
    }
    handleLines(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.polyline.drawPolylines(Object.assign({ polylinesMesh: inputs.babylonMesh, polylines: inputs.entity.map(e => ({ points: [e.start, e.end] })) }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.lines, options, result);
        return result;
    }
    handlePolylines(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.polyline.drawPolylines(Object.assign({ polylinesMesh: inputs.babylonMesh, polylines: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.polylines, options, result);
        return result;
    }
    handleVerbSurface(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.verbSurface.drawSurface(Object.assign({ surfaceMesh: inputs.babylonMesh, surface: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.verbSurface, options, result);
        return result;
    }
    handleVerbCurve(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.verbCurve.drawCurve(Object.assign({ curveMesh: inputs.babylonMesh, curve: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.verbCurve, options, result);
        return result;
    }
    handleNode(inputs) {
        let options = inputs.options ? inputs.options : this.defaultNodeOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = inputs.entity;
        this.node.drawNode(Object.assign({ node: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.node, options, result);
        return result;
    }
    handlePolyline(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.polyline.drawPolyline(Object.assign({ polylineMesh: inputs.babylonMesh, polyline: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.polyline, options, result);
        return result;
    }
    handlePoint(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.point.drawPoint(Object.assign({ pointMesh: inputs.babylonMesh, point: inputs.entity }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.point, options, result);
        return result;
    }
    handleLine(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        let result = this.polyline.drawPolylines(Object.assign({ polylinesMesh: inputs.babylonMesh, polylines: [{ points: [inputs.entity.start, inputs.entity.end] }] }, options));
        this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.line, options, result);
        return result;
    }
    handleJscadMeshes(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        return this.jscad.drawSolidOrPolygonMeshes(Object.assign({ jscadMesh: inputs.babylonMesh, meshes: inputs.entity }, options)).then(r => {
            this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.jscadMeshes, options, r);
            return r;
        });
    }
    handleOcctShape(inputs) {
        let options = inputs.options ? inputs.options : new Inputs.OCCT.DrawShapeDto(inputs.entity);
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        return this.occt.drawShape(Object.assign(Object.assign({ shape: inputs.entity }, new Inputs.Draw.DrawOcctShapeOptions()), options)).then(r => {
            this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.occt, options, r);
            return r;
        });
    }
    handleJscadMesh(inputs) {
        let options = inputs.options ? inputs.options : this.defaultBasicOptions;
        if (!inputs.options && inputs.babylonMesh && inputs.babylonMesh.metadata.options) {
            options = inputs.babylonMesh.metadata.options;
        }
        return this.jscad.drawSolidOrPolygonMesh(Object.assign({ jscadMesh: inputs.babylonMesh, mesh: inputs.entity }, options)).then(r => {
            this.applyGlobalSettingsAndMetadataAndShadowCasting(Inputs.Draw.drawingTypes.jscadMesh, options, r);
            return r;
        });
    }
    applyGlobalSettingsAndMetadataAndShadowCasting(type, options, result) {
        const typemeta = { type, options };
        const sgs = this.context.scene.metadata.shadowGenerators;
        result.isPickable = false;
        result.getChildMeshes().forEach(m => { m.isPickable = false; });
        if (sgs.length > 0) {
            result.receiveShadows = true;
            sgs.forEach(sg => sg.addShadowCaster(result));
            result.getChildMeshes().forEach(m => {
                m.receiveShadows = true;
                sgs.forEach(sg => sg.addShadowCaster(m));
            });
        }
        result.metadata = result.metadata ? Object.assign(Object.assign({}, result.metadata), typemeta) : typemeta;
    }
    detectPoint(entity) {
        return (Array.isArray(entity) && entity.length === 3 && this.checkIfElementsInArrayAreNumbers(entity));
    }
    detectPoints(entity) {
        return Array.isArray(entity) &&
            this.checkIfElementsInArrayAreArrays(entity) &&
            this.arraysInChildrenArraysContainNumbers(entity) &&
            this.arraysInChildrenArraysAreOfLength3(entity);
    }
    detectLine(entity) {
        return entity.start && entity.end && Array.isArray(entity.start) && Array.isArray(entity.end);
    }
    detectLines(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectLine(el));
    }
    detectPolyline(entity) {
        return entity.points && Array.isArray(entity.points);
    }
    detectPolylines(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectPolyline(el));
    }
    detectNode(entity) {
        return !Array.isArray(entity) && entity.id && entity.id.includes("node");
    }
    detectNodes(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectNode(el));
    }
    detectVerbCurve(entity) {
        return !Array.isArray(entity) && entity._data && entity._data.controlPoints && entity._data.knots && entity._data.degree;
    }
    detectVerbSurface(entity) {
        return !Array.isArray(entity) && entity._data && entity._data.controlPoints && entity._data.degreeU && entity._data.degreeV && entity._data.knotsU && entity._data.knotsV;
    }
    detectVerbCurves(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectVerbCurve(el));
    }
    detectVerbSurfaces(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectVerbSurface(el));
    }
    detectJscadMesh(entity) {
        return !Array.isArray(entity) && (entity.sides || entity.polygons);
    }
    detectJscadMeshes(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectJscadMesh(el));
    }
    detectOcctShape(entity) {
        return (entity === null || entity === void 0 ? void 0 : entity.type) === 'occ-shape';
    }
    detectTag(entity) {
        return !Array.isArray(entity) && entity.text;
    }
    detectTags(entity) {
        return Array.isArray(entity) && !entity.some(el => !this.detectTag(el));
    }
    checkIfElementsInArrayAreNumbers(array) {
        return !array.some(el => isNaN(el));
    }
    checkIfElementsInArrayAreArrays(array) {
        return !array.some(el => !Array.isArray(el));
    }
    arraysInChildrenArraysContainNumbers(array) {
        return !array.some(el => !this.checkIfElementsInArrayAreNumbers(el));
    }
    arraysInChildrenArraysAreOfLength3(array) {
        return !array.some(el => el.length !== 3);
    }
}
