var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Color3, Matrix, Mesh, MeshBuilder, PBRMetallicRoughnessMaterial, VertexData } from '@babylonjs/core';
import { JSCADBooleans } from './booleans';
import { JSCADExpansions } from './expansions';
import { JSCADExtrusions } from './extrusions';
import { JSCADHulls } from './hulls';
import { JSCADPath } from './path';
import { JSCADPolygon } from './polygon';
import { JSCADShapes } from './shapes';
import { JSCADText } from './text';
/**
 * Contains various functions for Solid meshes from JSCAD library https://github.com/jscad/OpenJSCAD.org
 * Thanks JSCAD community for developing this kernel
 */
export class JSCAD {
    constructor(jscadWorkerManager, context, geometryHelper) {
        this.jscadWorkerManager = jscadWorkerManager;
        this.context = context;
        this.geometryHelper = geometryHelper;
        this.booleans = new JSCADBooleans(jscadWorkerManager);
        this.expansions = new JSCADExpansions(jscadWorkerManager);
        this.extrusions = new JSCADExtrusions(jscadWorkerManager);
        this.hulls = new JSCADHulls(jscadWorkerManager);
        this.path = new JSCADPath(jscadWorkerManager);
        this.polygon = new JSCADPolygon(jscadWorkerManager);
        this.shapes = new JSCADShapes(jscadWorkerManager);
        this.text = new JSCADText(jscadWorkerManager);
    }
    /**
     * Draws a single solids
     * @param inputs Contains a solid or polygon and information for drawing
     * @returns Mesh that is being drawn by Babylon
     * @group jscad
     * @shortname draw solid
     * @ignore true
     */
    drawSolidOrPolygonMesh(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.jscadWorkerManager.genericCallToWorkerPromise('shapeToMesh', inputs);
            let meshToUpdate;
            if (inputs.jscadMesh && inputs.updatable) {
                meshToUpdate = inputs.jscadMesh;
            }
            else {
                meshToUpdate = new Mesh(`jscadMesh${Math.random()}`, this.context.scene);
            }
            const s = this.makeMesh(Object.assign(Object.assign({}, inputs), { colour: Array.isArray(inputs.colours) ? inputs.colours[0] : inputs.colours }), meshToUpdate, res);
            inputs.jscadMesh = s;
            return s;
        });
    }
    makeMesh(inputs, meshToUpdate, res) {
        this.createMesh(res.positions, res.indices, res.normals, meshToUpdate, res.transforms, inputs.updatable);
        meshToUpdate.material = new PBRMetallicRoughnessMaterial(`jscadMaterial${Math.random()}`, this.context.scene);
        meshToUpdate.flipFaces(false);
        const pbr = meshToUpdate.material;
        pbr.baseColor = Color3.FromHexString(inputs.colour);
        pbr.metallic = 1.0;
        pbr.roughness = 0.6;
        pbr.alpha = inputs.opacity;
        pbr.alphaMode = 1;
        pbr.backFaceCulling = true;
        pbr.zOffset = 0;
        meshToUpdate.isPickable = false;
        if (inputs.hidden) {
            meshToUpdate.isVisible = false;
        }
        return meshToUpdate;
    }
    /**
     * Draws multiple solids
     * @param inputs Contains solids or polygons and information for drawing
     * @returns Mesh that is being drawn by Babylon
     * @group jscad
     * @shortname draw solid
     * @ignore true
     */
    drawSolidOrPolygonMeshes(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jscadWorkerManager.genericCallToWorkerPromise('shapesToMeshes', inputs).then((res) => {
                let localOrigin;
                if (inputs.jscadMesh && inputs.updatable) {
                    localOrigin = inputs.jscadMesh;
                    const children = localOrigin.getChildMeshes();
                    children.forEach(mesh => { mesh.dispose(); localOrigin.removeChild(mesh); });
                }
                else {
                    localOrigin = MeshBuilder.CreateBox('local_origin' + Math.random(), { size: 1 }, this.context.scene);
                }
                localOrigin.isVisible = false;
                const colourIsArrayAndMatches = Array.isArray(inputs.colours) && inputs.colours.length === res.length;
                const colorsAreArrays = Array.isArray(inputs.colours);
                res.map((r, index) => {
                    let meshToUpdate = new Mesh(`jscadMesh${Math.random()}`, this.context.scene);
                    let colour;
                    if (colourIsArrayAndMatches) {
                        colour = inputs.colours[index];
                    }
                    else if (colorsAreArrays) {
                        colour = inputs.colours[0];
                    }
                    else {
                        colour = inputs.colours;
                    }
                    let m = this.makeMesh(Object.assign(Object.assign({}, inputs), { colour }), meshToUpdate, r);
                    m.parent = localOrigin;
                });
                inputs.jscadMesh = localOrigin;
                return localOrigin;
            });
        });
    }
    /**
     * Draws a 2D path
     * @param inputs Contains a path and information for drawing
     * @returns Mesh that is being drawn by Babylon
     * @group jscad
     * @shortname draw solid
     * @ignore true
     */
    drawPath(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                if (inputs.path.points) {
                    if (inputs.path.isClosed) {
                        const pt = inputs.path.points[0];
                        inputs.path.points.push([pt[0], 0, pt[1]]);
                    }
                }
                resolve(this.geometryHelper.drawPolyline(inputs.pathMesh, inputs.path.points, inputs.updatable, inputs.width, inputs.opacity, inputs.colour));
            });
        });
    }
    /**
     * Transforms the Jscad solid meshes with a given list of transformations.
     * @param inputs Solids with the transformation matrixes
     * @returns Solids with a transformation
     * @group transforms
     * @shortname transform solids
     * @drawable true
     */
    transformSolids(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jscadWorkerManager.genericCallToWorkerPromise('transformSolids', inputs);
        });
    }
    /**
     * Transforms the Jscad solid mesh with a given list of transformations.
     * @param inputs Solid with the transformation matrixes
     * @returns Solid with a transformation
     * @group transforms
     * @shortname transform solid
     * @drawable true
     */
    transformSolid(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jscadWorkerManager.genericCallToWorkerPromise('transformSolid', inputs);
        });
    }
    /**
     * Downloads the binary STL file from a 3D solid
     * @param inputs 3D Solid
     * @group io
     * @shortname solid to stl
     */
    downloadSolidSTL(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.jscadWorkerManager.genericCallToWorkerPromise('downloadSolidSTL', inputs);
            this.downloadSTL(res.blob, inputs.fileName);
        });
    }
    /**
     * Downloads the binary STL file from a 3D solids
     * @param inputs 3D Solid
     * @group io
     * @shortname solids to stl
     */
    downloadSolidsSTL(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.jscadWorkerManager.genericCallToWorkerPromise('downloadSolidsSTL', inputs);
            this.downloadSTL(res.blob, inputs.fileName);
        });
    }
    downloadSTL(blob, fileName) {
        const blobUrl = URL.createObjectURL(blob);
        const fileLink = document.createElement('a');
        fileLink.href = blobUrl;
        fileLink.target = '_self';
        fileLink.download = fileName + '.stl';
        fileLink.click();
        fileLink.remove();
    }
    createMesh(positions, indices, normals, jscadMesh, transforms, updatable) {
        const vertexData = new VertexData();
        vertexData.positions = positions;
        vertexData.indices = indices;
        VertexData.ComputeNormals(positions, indices, normals, { useRightHandedSystem: true });
        vertexData.normals = normals;
        vertexData.applyToMesh(jscadMesh, updatable);
        jscadMesh.setPreTransformMatrix(Matrix.FromArray(transforms));
    }
}
