import { Color3, Mesh, StandardMaterial, VertexBuffer, VertexData } from '@babylonjs/core';
/**
 * Contains various methods for points. Point in bitbybit is simply an array containing 3 numbers for [x, y, z].
 * Because of this form Point can be interchanged with Vector, which also is an array in [x, y, z] form.
 * When creating 2D points, z coordinate is simply set to 0 - [x, y, 0].
 */
export class Point {
    constructor(context, geometryHelper, line) {
        this.context = context;
        this.geometryHelper = geometryHelper;
        this.line = line;
    }
    /**
     * Draws a single point
     * @param inputs Contains a point to be drawn
     * @returns Mesh that is being drawn by Babylon
     * @group draw
     * @shortname point
     * @drawable false
     * @ignore true
     */
    drawPoint(inputs) {
        const vectorPoints = [inputs.point];
        let colours;
        if (Array.isArray(inputs.colours)) {
            colours = inputs.colours.map(colour => Color3.FromHexString(colour));
        }
        else {
            colours = [Color3.FromHexString(inputs.colours)];
        }
        const { positions, colors } = this.setUpPositionsAndColours(vectorPoints, colours);
        if (inputs.pointMesh && inputs.updatable) {
            this.updatePoints(inputs.pointMesh, inputs.opacity, inputs.size, positions, colors);
        }
        else {
            inputs.pointMesh = this.createNewMesh(`poinsMesh${Math.random()}`, positions, colors, inputs.opacity, inputs.size, inputs.updatable);
        }
        return inputs.pointMesh;
    }
    /**
     * Draws multiple points
     * @param inputs Contains a point array to be drawn
     * @returns Mesh that is being drawn by Babylon
     * @group draw
     * @shortname points sync
     * @drawable false
     * @ignore true
     */
    drawPoints(inputs) {
        const vectorPoints = inputs.points;
        let colours;
        if (Array.isArray(inputs.colours)) {
            colours = inputs.colours.map(colour => Color3.FromHexString(colour));
        }
        else {
            colours = [Color3.FromHexString(inputs.colours)];
        }
        const { positions, colors } = this.setUpPositionsAndColours(vectorPoints, colours);
        if (inputs.pointsMesh && inputs.updatable) {
            if (inputs.pointsMesh.getTotalVertices() === vectorPoints.length) {
                this.updatePoints(inputs.pointsMesh, inputs.opacity, inputs.size, positions, colors);
            }
            else {
                inputs.pointsMesh.dispose();
                inputs.pointsMesh = this.createNewMesh(`pointsMesh${Math.random()}`, positions, colors, inputs.opacity, inputs.size, inputs.updatable);
            }
        }
        else {
            inputs.pointsMesh = this.createNewMesh(`pointsMesh${Math.random()}`, positions, colors, inputs.opacity, inputs.size, inputs.updatable);
        }
        return inputs.pointsMesh;
    }
    /**
     * Draws multiple points async
     * @param inputs Contains a point array to be drawn
     * @returns Promise of a Mesh that will being drawn by Babylon
     * @group draw
     * @shortname points
     * @drawable false
     * @ignore true
     */
    drawPointsAsync(inputs) {
        return Promise.resolve(this.drawPoints(inputs));
    }
    /**
     * Transforms the single point
     * @param inputs Contains a point and the transformations to apply
     * @returns Transformed point
     * @group transforms
     * @shortname transform point
     * @drawable true
     */
    transformPoint(inputs) {
        const transformation = inputs.matrix;
        let transformedControlPoints = [inputs.point];
        transformedControlPoints = this.geometryHelper.transformControlPoints(transformation, transformedControlPoints);
        return transformedControlPoints[0];
    }
    /**
     * Transforms multiple points
     * @param inputs Contains points and the transformations to apply
     * @returns Transformed points
     * @group transforms
     * @shortname transform points
     * @drawable true
     */
    transformPoints(inputs) {
        return this.geometryHelper.transformControlPoints(inputs.matrix, inputs.points);
    }
    /**
     * Transforms multiple points by multiple transformations
     * @param inputs Contains points and the transformations to apply
     * @returns Transformed points
     * @group transforms
     * @shortname transforms for points
     * @drawable true
     */
    transformsForPoints(inputs) {
        if (inputs.points.length !== inputs.matrix.length) {
            throw new Error('You must provide equal nr of points and transformations');
        }
        return inputs.points.map((pt, index) => {
            return this.geometryHelper.transformControlPoints(inputs.matrix[index], [pt])[0];
        });
    }
    /**
     * Measures the closest distance between a point and a collection of points
     * @param inputs Point from which to measure and points to measure the distance against
     * @returns Distance to closest point
     * @group extract
     * @shortname distance to closest pt
     * @drawable false
     */
    closestPointFromPointsDistance(inputs) {
        return this.closestPointFromPointData(inputs).distance;
    }
    /**
     * Finds the closest point index between a point and a collection of points. Caution, index is not 0 based, it starts with 1.
     * @param inputs Point from which to find the index in a collection of points
     * @returns Closest point index
     * @group extract
     * @shortname index of closest pt
     * @drawable false
     */
    closestPointFromPointsIndex(inputs) {
        return this.closestPointFromPointData(inputs).index;
    }
    /**
     * Finds the closest point in a collection
     * @param inputs Point and points collection to find the closest point in
     * @returns Closest point
     * @group extract
     * @shortname closest pt
     * @drawable true
     */
    closestPointFromPoints(inputs) {
        return this.closestPointFromPointData(inputs).point;
    }
    /**
     * Finds the distance between two points
     * @param inputs Coordinates of start and end points
     * @returns Distance
     * @group measure
     * @shortname distance
     * @drawable false
     */
    distance(inputs) {
        return this.context.verb.core.Vec.dist(inputs.startPoint, inputs.endPoint);
    }
    /**
     * Multiply point by a specified amount
     * @param inputs The point to be multiplied and the amount of points to create
     * @returns Distance
     * @group transforms
     * @shortname multiply point
     * @drawable true
     */
    multiplyPoint(inputs) {
        const points = [];
        for (let i = 0; i < inputs.amountOfPoints; i++) {
            points.push([inputs.point[0], inputs.point[1], inputs.point[2]]);
        }
        return points;
    }
    /**
     * Get x coordinate of the point
     * @param inputs The point
     * @returns X coordinate
     * @group get
     * @shortname x coord
     * @drawable false
     */
    getX(inputs) {
        return inputs.point[0];
    }
    /**
     * Get y coordinate of the point
     * @param inputs The point
     * @returns Y coordinate
     * @group get
     * @shortname y coord
     * @drawable false
     */
    getY(inputs) {
        return inputs.point[1];
    }
    /**
     * Get z coordinate of the point
     * @param inputs The point
     * @returns Z coordinate
     * @group get
     * @shortname z coord
     * @drawable false
     */
    getZ(inputs) {
        return inputs.point[2];
    }
    /**
     * Get average point of points
     * @param inputs The points
     * @returns point
     * @group extract
     * @shortname average point
     * @drawable true
     */
    averagePoint(inputs) {
        const xVals = [];
        const yVals = [];
        const zVals = [];
        inputs.points.forEach(pt => {
            xVals.push(pt[0]);
            yVals.push(pt[1]);
            zVals.push(pt[2]);
        });
        return [
            xVals.reduce((p, c) => p + c, 0) / inputs.points.length,
            yVals.reduce((p, c) => p + c, 0) / inputs.points.length,
            zVals.reduce((p, c) => p + c, 0) / inputs.points.length,
        ];
    }
    /**
     * Creates the xyz point
     * @param inputs xyz information
     * @returns point 3d
     * @group create
     * @shortname point xyz
     * @drawable true
     */
    pointXYZ(inputs) {
        return [inputs.x, inputs.y, inputs.z];
    }
    /**
     * Creates the xy point
     * @param inputs xy information
     * @returns point 3d
     * @group create
     * @shortname point xy
     * @drawable false
     */
    pointXY(inputs) {
        return [inputs.x, inputs.y];
    }
    /**
     * Creates the spiral out of multiple points
     * @param inputs Spiral information
     * @returns Specified number of points in the array along the spiral
     * @group create
     * @shortname spiral
     * @drawable true
     */
    spiral(inputs) {
        const phi = inputs.phi;
        const b = Math.log(phi) / (Math.PI / inputs.widening);
        const spiral = [];
        const step = inputs.radius / inputs.numberPoints;
        for (let i = 0; i < inputs.radius; i += step) {
            const th = Math.log(i / inputs.factor) / b;
            const x = i * Math.cos(th);
            const y = i * Math.sin(th);
            spiral.push([x ? x : 0, y ? y : 0, 0]);
        }
        return spiral;
    }
    /**
     * Creates a flat point grid on XY plane. This grid contains center points for hexagons of the given radius.
     * Be aware that we control only the nr of hexagons to be made and not the length and width of the grid.
     * @param inputs Information about hexagon and the grid
     * @returns Points in the array on the grid
     * @group create
     * @shortname hex grid
     * @drawable true
     */
    hexGrid(inputs) {
        const xLength = Math.sqrt(Math.pow(inputs.radiusHexagon, 2) - Math.pow(inputs.radiusHexagon / 2, 2));
        const points = [];
        for (let ix = 0; ix < inputs.nrHexagonsX; ix++) {
            const coordX = ix * xLength * 2;
            for (let iy = 0; iy < inputs.nrHexagonsY; iy++) {
                const coordY = (inputs.radiusHexagon + inputs.radiusHexagon / 2) * iy;
                points.push([coordX + (iy % 2 === 0 ? 0 : xLength), coordY, 0]);
            }
        }
        return points;
    }
    closestPointFromPointData(inputs) {
        let distance = Number.MAX_SAFE_INTEGER;
        let closestPointIndex;
        let point;
        for (let i = 0; i < inputs.points.length; i++) {
            const pt = inputs.points[i];
            const currentDist = this.context.verb.core.Vec.dist(inputs.point, pt);
            if (currentDist < distance) {
                distance = currentDist;
                closestPointIndex = i;
                point = pt;
            }
        }
        return { index: closestPointIndex + 1, distance, point };
    }
    createNewMesh(meshName, positions, colors, opacity, size, updatable) {
        const vertexData = new VertexData();
        vertexData.positions = positions;
        vertexData.colors = colors;
        const pointsMesh = new Mesh(meshName, this.context.scene);
        vertexData.applyToMesh(pointsMesh, updatable);
        const mat = new StandardMaterial(`mat${Math.random()}`, this.context.scene);
        mat.emissiveColor = new Color3(1, 1, 1);
        mat.disableLighting = true;
        mat.pointsCloud = true;
        mat.alpha = opacity;
        mat.pointSize = size;
        pointsMesh.material = mat;
        return pointsMesh;
    }
    updatePoints(mesh, opacity, size, positions, colors) {
        mesh.updateVerticesData(VertexBuffer.PositionKind, positions);
        mesh.updateVerticesData(VertexBuffer.ColorKind, colors);
        mesh.material.alpha = opacity;
        mesh.material.pointSize = size;
    }
    setUpPositionsAndColours(vectorPoints, colours) {
        const positions = [];
        const colors = [];
        if (colours.length === vectorPoints.length) {
            vectorPoints.forEach((p, index) => {
                positions.push(...p);
                colors.push(colours[index].r, colours[index].g, colours[index].b, 1);
            });
        }
        else {
            vectorPoints.forEach((p, index) => {
                positions.push(...p);
                colors.push(colours[0].r, colours[0].g, colours[0].b, 1);
            });
        }
        return { positions, colors };
    }
}
