import { Angle } from '@babylonjs/core';
/**
 * Contains various functions for Path from JSCAD library https://github.com/jscad/OpenJSCAD.org
 * Thanks JSCAD community for developing this kernel
 */
export class JSCADPath {
    constructor(jscad, vecHelper) {
        this.jscad = jscad;
        this.vecHelper = vecHelper;
    }
    createFromPoints(inputs) {
        const twoDimensionalPoints = inputs.points.map(pt => [pt[0], pt[1]]);
        return this.removeDuplicatesAndCreateFromPoints(twoDimensionalPoints, inputs.closed);
    }
    createFromPolyline(inputs) {
        const twoDimensionalPoints = inputs.polyline.points.map(pt => [pt[0], pt[1]]);
        return this.removeDuplicatesAndCreateFromPoints(twoDimensionalPoints, inputs.closed);
    }
    createFromCurve(inputs) {
        const twoDimensionalPoints = inputs.curve.tessellate().map(pt => [pt[0], pt[1]]);
        return this.removeDuplicatesAndCreateFromPoints(twoDimensionalPoints, inputs.closed);
    }
    createEmpty() {
        return this.jscad.geometries.path2.create();
    }
    close(inputs) {
        return this.jscad.geometries.path2.close(inputs.path);
    }
    appendPoints(inputs) {
        const twoDimensionalPoints = inputs.points.map(pt => [pt[0], pt[1]]);
        const duplicatePointsRemoved = this.vecHelper.removeConsecutiveDuplicates(twoDimensionalPoints);
        return this.jscad.geometries.path2.appendPoints(duplicatePointsRemoved, inputs.path);
    }
    appendPolyline(inputs) {
        const twoDimensionalPoints = inputs.polyline.points.map(pt => [pt[0], pt[1]]);
        return this.appendPoints({ points: twoDimensionalPoints, path: inputs.path });
    }
    appendCurve(inputs) {
        const twoDimensionalPoints = inputs.curve.tessellate().map(pt => [pt[0], pt[1]]);
        return this.appendPoints({ points: twoDimensionalPoints, path: inputs.path });
    }
    appendArc(inputs) {
        const endpoint = [inputs.endPoint[0], inputs.endPoint[1]];
        const radius = [inputs.radiusX, inputs.radiusY];
        return this.jscad.geometries.path2.appendArc({
            endpoint,
            radius,
            xaxisrotation: Angle.FromDegrees(inputs.xAxisRotation).radians(),
            clockwise: inputs.clockwise,
            large: inputs.large,
            segments: inputs.segments,
        }, inputs.path);
    }
    removeDuplicatesAndCreateFromPoints(twoDimensionalPoints, closed) {
        const duplicatePointsRemoved = this.vecHelper.removeConsecutiveDuplicates(twoDimensionalPoints);
        let path2d = this.jscad.geometries.path2.fromPoints({}, duplicatePointsRemoved);
        if (closed) {
            path2d = this.jscad.geometries.path2.close(path2d);
        }
        return path2d;
    }
}
